#include "rumd_data.h"
#include <cassert>

class rumd_msd {
public:
  rumd_msd();
  ~rumd_msd() { 
    FreeArrays();
    if(num_moleculetypes > 0)
      FreeArraysCM();
  }
  void SetVerbose(bool v) { verbose = v; }
  void SetDirectory(const std::string& directory) {this->directory = directory;}

  void ComputeAll(unsigned int first_block=0, int last_block=-1, unsigned int particlesPerMol = 1);

  void Copy_MSD_To_Array(double (*msd_array)[2], unsigned int type);
  void Copy_ISF_To_Array(double (*isf_array)[2], unsigned int type);
  void Copy_Alpha2_To_Array(double (*alpha2_array)[2], unsigned int type);

  void WriteMSD(const std::string& filename);
  void WriteMSD_CM(const std::string& filename);
  void WriteAlpha2(const std::string& filename);
  void WriteFs(const std::string& filename);
  void WriteFS_CM(const std::string& filename);
  void WriteFS_SQ(const std::string& filename);
  
  unsigned int GetMaxDataIndex() const { return MaxDataIndex; }

  private:
  rumd_msd(const rumd_msd&);
  rumd_msd& operator=(const rumd_msd&);

  void AllocateArrays(unsigned int set_MaxDataIndex, unsigned int set_num_types);
  void AllocateArraysCM(unsigned int ppm);
  void FreeArrays();
  void FreeArraysCM();
  void ResetArrays();
  void CalcR2(Conf *C0, Conf *Ct, double *R2,  double *R4, double *Fs, double *Fs2, unsigned int num_types, double *q);
  void CalcR2cm(Conf *C0, Conf *Ct, double *R2,  double *R4, double *Fs, unsigned int num_types, double *q);

  std::string directory;
  Conf C0;
  Conf C1;
  // Setup arrays to collect statistics
  unsigned long int *Time;
  unsigned long int *Count;
  double **R2;
  double **R4;
  double **Fs;
  double **Fs2;

  double **R2cm;
  double **R4cm;
  double **Fscm;

  double *Fs_;
  double *qvalues;
  int *numParticlesOfType;
  
  unsigned int MaxDataIndex;
  unsigned int num_types;
  unsigned int num_particles;
  unsigned int particlesPerMol;
  unsigned int num_moleculetypes;

  bool verbose;
  unsigned int numFilenameDigits;
  float dt;
};
