#include "rumd_msd.h"
#include "rumd/RUMD_Error.h"
#include <iostream>


int main(int argc, char *argv[])
{  
  unsigned int first_block=0;
  unsigned int last_block = -1;   
  unsigned int particlesPerMol = 0;
  bool verbose = false;
  std::string directory("TrajectoryFiles");

  static const char *usage = "Usage: %s [-h] [-m <particles per molecule>] [-f <first_block>] [-l <last_block>] [-d directory] [-v <verbose>]\n";
  int opt;
  while ((opt = getopt(argc, argv, "hm:f:l:d:v:")) != -1) {
    switch (opt) {
    case 'm':
      particlesPerMol = atoi(optarg);
      break;
    case 'h':
      fprintf(stdout, usage, argv[0]);
      exit(EXIT_SUCCESS);
    case 'f':
      first_block = atoi(optarg);
      break;
    case 'l':
      last_block = atoi(optarg);
      break;
    case 'd':
      directory = std::string(optarg);
      break;
    case 'v':
      verbose = (bool) atoi(optarg);
      break;
    case '?':
      fprintf(stderr, "%s: unknown option: -%c\n", argv[0], optopt);
      fprintf(stderr, usage, argv[0]);
      exit(EXIT_FAILURE);
      break;
    case ':':
      fprintf(stderr, "%s: missing option argument for option: -%c\n", argv[0], optopt);
      fprintf(stderr, usage, argv[0]);
      exit(EXIT_FAILURE);
      break;
    default:
      fprintf(stderr, usage, argv[0]);
      exit(EXIT_FAILURE);
    }
  }

  try {
    rumd_msd msd;
    msd.SetVerbose(verbose);
    msd.SetDirectory(directory);
    msd.ComputeAll(first_block, last_block, particlesPerMol);

    msd.WriteMSD("msd.dat");
    
    msd.WriteAlpha2("alpha2.dat");
    msd.WriteFs("Fs.dat");

    msd.WriteFS_SQ("FsSq.dat");

    if (particlesPerMol>1) {
      msd.WriteMSD_CM("msd_cm.dat");
      msd.WriteFS_CM("Fs_cm.dat");
    }

  }
  catch(RUMD_Error e){
    std::cerr << "RUMD_Error thrown from function " << e.className << "::" << e.methodName << ": " << std::endl << e.errorStr << std::endl;
    return 1;
  }
  return 0;

}
